public inbox for devel@edk2.groups.io
 help / color / mirror / Atom feed
From: "Min Xu" <min.m.xu@intel.com>
To: devel@edk2.groups.io
Cc: Min Xu <min.m.xu@intel.com>,
	Michael D Kinney <michael.d.kinney@intel.com>,
	Liming Gao <gaoliming@byosoft.com.cn>,
	Zhiguang Liu <zhiguang.liu@intel.com>,
	Jiewen Yao <jiewen.yao@intel.com>,
	Jian J Wang <jian.j.wang@intel.com>,
	Sami Mujawar <sami.mujawar@arm.com>
Subject: [PATCH V3 3/3] SecurityPkg: Support TeeMeasurementProtocol in DxeTpmMeasurementLib
Date: Thu, 28 Oct 2021 12:59:29 +0800	[thread overview]
Message-ID: <a83c7ce46b6ef706000d61d3658409f2d1a15c64.1635395810.git.min.m.xu@intel.com> (raw)
In-Reply-To: <cover.1635395810.git.min.m.xu@intel.com>

BZ: https://bugzilla.tianocore.org/show_bug.cgi?id=3625

DxeTpmMeasurementLib supports TPM based measurement in DXE phase.
After TeeMeasurementProtocol is introduced, TD based measurement needs
to be supported in DxeTpmMeasurementLib as well.

In TpmMeasureAndLogData, TEE based measurement will be first called.
If it failed, TPM based measurement will be called sequentially.
Currently there is an assumption that TEE based measurement and
TPM based measurement won't be exist at the same time.If the
assumption is not true in the future, we will revisit here then.

Cc: Michael D Kinney <michael.d.kinney@intel.com>
Cc: Liming Gao <gaoliming@byosoft.com.cn>
Cc: Zhiguang Liu <zhiguang.liu@intel.com>
Cc: Jiewen Yao <jiewen.yao@intel.com>
Cc: Jian J Wang <jian.j.wang@intel.com>
Cc: Sami Mujawar <sami.mujawar@arm.com>
Signed-off-by: Min Xu <min.m.xu@intel.com>
---
 .../DxeTpmMeasurementLib.c                    | 88 ++++++++++++++++++-
 .../DxeTpmMeasurementLib.inf                  |  5 +-
 2 files changed, 88 insertions(+), 5 deletions(-)

diff --git a/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.inf b/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.inf
index 7d41bc41f95d..501d5d66e0fb 100644
--- a/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.inf
+++ b/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.inf
@@ -40,5 +40,6 @@
   UefiBootServicesTableLib
 
 [Protocols]
-  gEfiTcgProtocolGuid           ## SOMETIMES_CONSUMES
-  gEfiTcg2ProtocolGuid          ## SOMETIMES_CONSUMES
+  gEfiTcgProtocolGuid               ## SOMETIMES_CONSUMES
+  gEfiTcg2ProtocolGuid              ## SOMETIMES_CONSUMES
+  gEfiTeeMeasurementProtocolGuid    ## SOMETIMES_CONSUMES
diff --git a/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.c b/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.c
index 061136ee7860..1b856cfc7a0d 100644
--- a/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.c
+++ b/SecurityPkg/Library/DxeTpmMeasurementLib/DxeTpmMeasurementLib.c
@@ -19,7 +19,8 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 
 #include <Guid/Acpi.h>
 #include <IndustryStandard/Acpi.h>
-
+#include <Protocol/TeeMeasurement.h>
+#include <Protocol/TdProtocol.h>
 
 
 /**
@@ -149,6 +150,73 @@ Tpm20MeasureAndLogData (
   return Status;
 }
 
+/**
+  Tee measure and log data, and extend the measurement result into a
+  specific TEE MR.
+
+  @param[in]  PcrIndex         PCR Index.
+  @param[in]  EventType        Event type.
+  @param[in]  EventLog         Measurement event log.
+  @param[in]  LogLen           Event log length in bytes.
+  @param[in]  HashData         The start of the data buffer to be hashed, extended.
+  @param[in]  HashDataLen      The length, in bytes, of the buffer referenced by HashData
+
+  @retval EFI_SUCCESS           Operation completed successfully.
+  @retval EFI_UNSUPPORTED       Tdx device not available.
+  @retval EFI_OUT_OF_RESOURCES  Out of memory.
+  @retval EFI_DEVICE_ERROR      The operation was unsuccessful.
+**/
+EFI_STATUS
+EFIAPI
+TeeMeasureAndLogData (
+  IN UINT32             PcrIndex,
+  IN UINT32             EventType,
+  IN VOID               *EventLog,
+  IN UINT32             LogLen,
+  IN VOID               *HashData,
+  IN UINT64             HashDataLen
+  )
+{
+  EFI_STATUS                    Status;
+  EFI_TEE_MEASUREMENT_PROTOCOL  *TeeProtocol;
+  EFI_TEE_EVENT                 *EfiTeeEvent;
+  UINT32                        MrIndex;
+
+  Status = gBS->LocateProtocol (&gEfiTeeMeasurementProtocolGuid, NULL, (VOID **) &TeeProtocol);
+  if (EFI_ERROR (Status)) {
+    return Status;
+  }
+
+  Status = TeeProtocol->MapPcrToMrIndex (TeeProtocol, PcrIndex, &MrIndex);
+  if (EFI_ERROR (Status)) {
+    return EFI_INVALID_PARAMETER;
+  }
+
+  EfiTeeEvent = (EFI_TEE_EVENT *) AllocateZeroPool (LogLen + sizeof (EFI_TEE_EVENT));
+  if(EfiTeeEvent == NULL) {
+    return EFI_OUT_OF_RESOURCES;
+  }
+
+  EfiTeeEvent->Size = (UINT32) LogLen + sizeof (EFI_TEE_EVENT) - sizeof (EfiTeeEvent->Event);
+  EfiTeeEvent->Header.HeaderSize    = sizeof (EFI_TEE_EVENT_HEADER);
+  EfiTeeEvent->Header.HeaderVersion = EFI_TEE_EVENT_HEADER_VERSION;
+  EfiTeeEvent->Header.MrIndex       = MrIndex;
+  EfiTeeEvent->Header.EventType     = EventType;
+  CopyMem (&EfiTeeEvent->Event[0], EventLog, LogLen);
+
+  Status = TeeProtocol->HashLogExtendEvent (
+                           TeeProtocol,
+                           0,
+                           (EFI_PHYSICAL_ADDRESS) (UINTN) HashData,
+                           HashDataLen,
+                           EfiTeeEvent
+                           );
+  FreePool (EfiTeeEvent);
+
+  return Status;
+}
+
+
 /**
   Tpm measure and log data, and extend the measurement result into a specific PCR.
 
@@ -178,9 +246,9 @@ TpmMeasureAndLogData (
   EFI_STATUS  Status;
 
   //
   // Try to measure using Tpm20 protocol
   //
-  Status = Tpm20MeasureAndLogData(
+  Status = TeeMeasureAndLogData (
              PcrIndex,
              EventType,
              EventLog,
@@ -189,6 +257,20 @@ TpmMeasureAndLogData (
              HashDataLen
              );
 
+  if (EFI_ERROR (Status)) {
+    //
+    // Try to measure using Tpm20 protocol
+    //
+    Status = Tpm20MeasureAndLogData(
+               PcrIndex,
+               EventType,
+               EventLog,
+               LogLen,
+               HashData,
+               HashDataLen
+               );
+  }
+
   if (EFI_ERROR (Status)) {
     //
     // Try to measure using Tpm1.2 protocol
@@ -178,9 +246,9 @@ TpmMeasureAndLogData (
 
   //
-  // Try to measure using Tpm20 protocol
+  // Try to measure using Tee measurement protocol
   //
-- 
2.29.2.windows.2


      parent reply	other threads:[~2021-10-28  5:00 UTC|newest]

Thread overview: 6+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2021-10-28  4:59 [PATCH V3 0/3] Introduce TeeMeasurementProtocol into EDK2 Min Xu
2021-10-28  4:59 ` [PATCH V3 1/3] MdePkg: Introduce TeeMeasurementProtocol for TEE Guest firmware Min Xu
2021-10-29  2:06   ` 回复: [edk2-devel] " gaoliming
2021-10-29  2:55     ` Min Xu
2021-10-28  4:59 ` [PATCH V3 2/3] SecurityPkg: Support TeeMeasurementProtocol in DxeTpm2MeasureBootLib Min Xu
2021-10-28  4:59 ` Min Xu [this message]

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-list from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=a83c7ce46b6ef706000d61d3658409f2d1a15c64.1635395810.git.min.m.xu@intel.com \
    --to=devel@edk2.groups.io \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox